import java.util.*;

public abstract class Dealer implements Observer {
	private String id;
	private CardDeck cardDeck;
	private Table table;
	
	public Dealer(String id) {
		setId(id);
		cardDeck = new CardDeck();
		table = null;
	}	

	public void startHand(double ante, double smallBlind, double bigBlind) {
            TableGui gui = TableGui.getTableGui();
            prepareDeck();
            setPlayerHandMucks(false);
            gui.muckAllCards();
            getTable().setCurrentBet(bigBlind);
            if(ante > 0)
		getAntes(ante);
            if(bigBlind > 0)
		getBlinds(smallBlind, bigBlind);
            gui.updatePlayerChips();
            dealToHands(5);  //default: deals 5 cards to each hand
            gui.startPreflop();
	}
        
	public void endHand() {
		Table table = getTable();
		determineWinners();
		setPlayerHandMucks(true);
                table.incrementButton();
                table.clearPots();
                TableGui gui = TableGui.getTableGui();
                gui.enableStartButton();
	}
	
	public void prepareDeck() {
		setCardDeck(new CardDeck());  //creates a new card deck
		getCardDeck().shuffleDeck();
	}

	//sets the mucked value of all hands to true or false
	public void setPlayerHandMucks(boolean muck) {
		Table dealTable = getTable();
		int numHands = dealTable.getNumPlayerHands();
		for(int i = 0; i < numHands; i++) {
			PlayerHand pH = dealTable.getPlayerHand(i);
			pH.setMucked(muck);
		}
	}
	
	public void dealToHands(int numCards) {
		CardDeck deck = getCardDeck();
		Table table = getTable();
		if(deck.getNumCards() != CardDeck.DECKSIZE) {  //sanity check that it is ok to deal a new hand
			System.out.println("Cannot deal new hand right now, the deck is not full!");
			return;
		}
		int numPlayerHands = table.getNumPlayerHands();
                int totalCards = numPlayerHands * numCards;
		for(int i = 1; i <= totalCards; i++) {  //start with the hand after the button                   
        		Card topCard = deck.getTopCard();
			if(topCard != null) {  //if there are still cards in the deck
                                try{ 
                                    Thread.sleep(200);
                                } catch (InterruptedException e) {}
                                int handPos = i % numPlayerHands;
				table.addCardToHand(topCard, handPos);
                                int buttonPos = table.getButtonPosition();
                                TableGui gui = TableGui.getTableGui();
                                gui.setPlayerCard((i + buttonPos) % totalCards, topCard.getValue() + topCard.getSuit());
			}
			else {
				System.out.println("Ran out of cards in deck - this should not have happened! =( =( =(");
				return;
			}
		}
	}
        
	
        //TODO: not implemented in GUI yet
	public void getAntes(double ante) {
		for(int i = 0; i < getTable().getNumPlayerHands(); i++) {
			PlayerHand pHand = getTable().getPlayerHand(i);
			Player player = pHand.getPlayer();
			PotContribution pC = player.bet(ante, true);
			getTable().addPotContribution(pC, false);
		}		
	}
	
	public void getBlinds(double smallBlind, double bigBlind) {
		PlayerHand smallBHand = getTable().getPlayerHand(1);
		Player sPlayer = smallBHand.getPlayer();
		PlayerHand bigBHand = getTable().getPlayerHand(2);
		Player bPlayer = bigBHand.getPlayer();
                PotContribution pCBig = bPlayer.bet(bigBlind, true);
                PotContribution pCSmall = sPlayer.bet(smallBlind, true);
                getTable().addPotContribution(pCSmall, false);
                getTable().addPotContribution(pCBig, false);
                TableGui gui = TableGui.getTableGui();
                gui.displayBlinds(smallBlind, bigBlind);
	}
        
        //TODO: change to determine winners of each pot
        public void determineWinners() {
            List<PlayerHand> winners = new ArrayList<PlayerHand>();
            List<PlayerHand> hands = getTable().getPlayerHands();
            for(int i = 0; i < hands.size(); i++) {
            	PlayerHand pH = hands.get(i);
            	if(!pH.getMucked())  //only count hands that haven't been folded
                	winners.add(pH);			
		}
            Collections.sort(winners);  //sort so winners are first in list
            PlayerHand pH = winners.get(0);
            //System.out.println();
            //System.out.println(pH.getPlayer().getName() + " wins with: " + pH.getRankedHand().getRank());
            TableGui gui = TableGui.getTableGui();
            gui.setStatusText(pH.getPlayer().getName() + " wins with: " + pH.getRankedHand().rankToString() + "\n");
            for(int i = 1; i < winners.size(); i++) {
                PlayerHand pH2 = winners.get(i);
                if(pH.compareTo(pH2) == 0) {  //check for equal hands (split pot) 
                    gui.setStatusText(pH2.getPlayer().getName() + " wins with: " + pH2.getRankedHand().rankToString() + "\n");
                }
		//System.out.println(pH2.getPlayer().getName() + " wins with: " + pH2.getRankedHand().getRank());
		//else {
                    //System.out.println(pH2.getPlayer().getName() + " had: " + pH2.getRankedHand().getRank());
                    //break;
		//}				
            }	
	}
	
	public CardDeck getCardDeck() {
		return cardDeck;
	}
	
	public void setCardDeck(CardDeck deck) {
		cardDeck = deck;
	}
        
        private void setId(String id) {
            this.id = id;
        }
	
	public Table getTable() {
		return table;
	}
	
	public void setTable(Table table) {
		this.table = table;
	}
        
        public abstract void update(Observable o, Object arg);
}
