import java.util.*;

public class PlayerHand implements Comparable {
	Player player;
	boolean mucked;
	Set<Card> cards;
		
	public PlayerHand(Player player) {
		this.player = player;
		setMucked(false);
		cards = new TreeSet<Card>();
	}
	
	public void addCard(Card card) {
		cards.add(card);
	}
	
	public void removeCard(Card card) {
		cards.remove(card);
	}
	
	private Set<Card> getCards() {
		return cards;
	}
	
	public Player getPlayer() {
		return player;
	}
	
	public void setMucked(boolean muck) {
		if(muck) {
                    cards.clear();
                    mucked = true;
		} else {
                    mucked = false;
		}
	}
	
	public boolean getMucked() {
		return mucked;
	}
	
	public int getNumCards() {
		return cards.size();
	}
	
	public RankedHand getRankedHand() {
		//CardAnalyser regAna = CardAnalyser.getCardAnalyser(CardAnalyser.REGULAR);
                CardAnalyser regAna = new RegularCardAnalyser();
		RankedHand hand = regAna.analyseCards(getCards());
		return hand;
	}
	
	public int compareTo(Object o) {
		assert o instanceof PlayerHand;
		PlayerHand hand = (PlayerHand) o;
		RankedHand rHand = hand.getRankedHand();
		return rHand.compareTo(getRankedHand());
	}
	
	//PlayerHands are equal if they have the same Player
	public boolean equals(Object o) {
		if(o instanceof PlayerHand) {
			PlayerHand playerHand = (PlayerHand) o;
			Player pl = playerHand.getPlayer();
			return pl.equals(getPlayer());
		}
		else {
			return false;
		}
	}
	
	public int hashCode() {
		return getPlayer().hashCode();
	}
}
