/*
 * NewJFrame.java
 *
 * Created on 28 September 2008, 13:22
 */

/**
 *
 * @author  ejf31
 */

import java.util.*;
import java.text.*;
import javax.swing.*;

public class TableGui extends javax.swing.JFrame {
    
    private static TableGui gui;
    private static PokerGame pokerGame;  //the game that the gui is displaying
    private Table table;
    private List<JLabel> pCards = new ArrayList<JLabel>();
    private List<JLabel> cCards = new ArrayList<JLabel>();
    private List<JTextPane> betTexts = new ArrayList<JTextPane>();
    private List<JTextPane> playerTexts = new ArrayList<JTextPane>();
    private List<JTextPane> playerChipTexts = new ArrayList<JTextPane>();
    private int playersTurn;  //which players turn it is relative to button, button = 0
    private List<Player> players;
    private boolean afterFlop;
    private int commCardCounter;
    private boolean handStarted;
    
    private TableGui() {
        initComponents();
        pCards.add(p1c1);
        pCards.add(p2c1);
        pCards.add(p3c1);
        pCards.add(p4c1);
        pCards.add(p1c2);
        pCards.add(p2c2);
        pCards.add(p3c2);
        pCards.add(p4c2);
        addCommCards();
        addBetTexts();
        addPlayerTexts();
        addPlayerChipTexts();
        playersTurn = 3;
        afterFlop = false;
        handStarted = false;
        commCardCounter = 0;
    }
    
    public static TableGui getTableGui() {
        if(gui == null) {
            gui = new TableGui();
        }
        return gui;
    }
    

    public void incrementPlayersTurn() {
        playersTurn = (playersTurn + 1) % Table.MAX_PLAYERS;
        Player player = players.get(playersTurn);
        if(player.getPlayerHand().getMucked()) {  //skip over players who have mucked
            incrementPlayersTurn();
            return;
        }
        startTurn(playersTurn);
    }
    
    private void resetPlayerTexts() {
        for(int i = 1; i <= playerTexts.size(); i++) {
            JTextPane tempText = playerTexts.get(i - 1);
            tempText.setText("Player" + i);            
        }        
    }
    
    public void startTurn(int playerPos) {
        Player player = players.get(playerPos);
        double currentBet = getTable().getCurrentBet();
        setBetTextAmount(currentBet);
        checkButton.setEnabled(true);
        callButton.setEnabled(true);
        resetPlayerTexts();
        JTextPane playerText = playerTexts.get(playerPos);
        playerText.setText(playerText.getText() + " *");        
        TablePot tableP = getTable().getActivePot();
        if(tableP == null) {
            System.out.println("Error: No active pot at start of turn, there should always be an active pot!");
            System.exit(1);
        }
        PotContribution potC = tableP.getPlayerContribution(player);
        if(potC != null) {
            double contAmount = potC.getAmount();
            if(currentBet > contAmount)
                checkButton.setEnabled(false);
            else if(currentBet == contAmount)
                callButton.setEnabled(false);
        } else if(currentBet > 0) {
            checkButton.setEnabled(false);
        }
        if(getTable().getCurrentBet() == 0)
            callButton.setEnabled(false);
    }
    
    private String doubleTwoDecimalPlaces(double num) {
        DecimalFormat twoDigit = new DecimalFormat("#,##0.00");  //formats to 2 decimal places
        return twoDigit.format(num);
    }
    
    public void setBetTextAmount(double amount) {
        betAmount.setText(doubleTwoDecimalPlaces(amount));
    }
    
    public void updatePotDisplay(double amount) {
        potAmount.setText(doubleTwoDecimalPlaces(amount));
    }
    
    private void clearPotDisplay() {
        potAmount.setText("");
    }
    
    public void updatePlayerChips() {
        for(int i = 0; i < playerChipTexts.size(); i++) {
            Player player = players.get(i);
            JTextPane chipText = playerChipTexts.get(i);  //get the equivalent chip text for that Player
            chipText.setText(doubleTwoDecimalPlaces(player.getChips()));
        }      
    }
    
    //TODO: how to determine when everyone else has folded?
    
    private Player getCurrentPlayer() {
        return players.get(playersTurn);
    }
    
    public void setPlayers(List<Player> players) {
        this.players = players;
    }
    
    public void startPreflop() {
        handStarted = true;
        playersTurn = (table.getButtonPosition() + 3) % getTable().getNumPlayerHands();  //start after blinds
        startTurn(playersTurn);
        setBetTextAmount(pokerGame.getBigBlind());
    }
    
    public void startNextBetting() {
        afterFlop = true;
        clearBetTexts();
        setBetTextAmount(pokerGame.getBigBlind());
        playersTurn = table.getButtonPosition();  
        incrementPlayersTurn();  //start directly after button
    }

    
    private void addCommCards() {
        cCards.add(commCard1);
        cCards.add(commCard2);
        cCards.add(commCard3);
        cCards.add(commCard4);
        cCards.add(commCard5);
    }
    
    private void addBetTexts() {
        betTexts.add(p1bet);
        betTexts.add(p2bet);
        betTexts.add(p3bet);
        betTexts.add(p4bet);
    }
    
    private void clearBetTexts() {
        for(int i = 0; i < betTexts.size(); i++) {
            JTextPane betText = betTexts.get(i);
            betText.setText("");
        }
    }
    
    private void addPlayerTexts() {
        playerTexts.add(Player1);
        playerTexts.add(Player2);
        playerTexts.add(Player3);
        playerTexts.add(Player4);
    }
    
    private void addPlayerChipTexts() {
        playerChipTexts.add(player1Chips);
        playerChipTexts.add(player2Chips);
        playerChipTexts.add(player3Chips);
        playerChipTexts.add(player4Chips);        
    }
    
    public void setPlayerCard(int cardPos, String cardFile) {
        JLabel tempCard = pCards.get(cardPos);
        tempCard.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/" + cardFile + ".jpg"));     
    }
    
    public void addCommCard(String cardFile) {
        JLabel tempCard = cCards.get(commCardCounter);
        if(tempCard != null) {
            tempCard.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/commCards/" + cardFile + ".jpg"));           
            commCardCounter++;
        }
    }
    
    public void setStatusText(String text) {
        statusText.setText(statusText.getText() + " " + text);    
    }
    
    private void clearStatusText() {
        statusText.setText("");
    }
    
    public void muckAllCards() {
        for(int i = 0; i < pCards.size(); i++) {
            JLabel tempCard = pCards.get(i);
            tempCard.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));
        }
        for(int j = 0; j < cCards.size(); j++) {
            JLabel tempCard = cCards.get(j);
            tempCard.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/commCards/back.jpg"));
        }
    }
    
    public void displayBlinds(double small, double big) {
        int numPlayers = table.getNumPlayerHands();
        int smallBlindPos = (table.getButtonPosition() + 1) % numPlayers;
        JTextPane smallBText = betTexts.get(smallBlindPos);
        JTextPane bigBText = betTexts.get((smallBlindPos + 1) % numPlayers);
        smallBText.setText(doubleTwoDecimalPlaces(small));
        bigBText.setText(doubleTwoDecimalPlaces(big));
    }
    
    public static void setPokerGame(PokerGame game) {
        pokerGame = game;
    }
    
    public void setTable(Table table) {
        this.table = table;
    }
    
    private Table getTable() {
        return table;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        p1c1 = new javax.swing.JLabel();
        p1c2 = new javax.swing.JLabel();
        p2c1 = new javax.swing.JLabel();
        p2c2 = new javax.swing.JLabel();
        p3c1 = new javax.swing.JLabel();
        p3c2 = new javax.swing.JLabel();
        p4c1 = new javax.swing.JLabel();
        p4c2 = new javax.swing.JLabel();
        foldButton = new javax.swing.JButton();
        callButton = new javax.swing.JButton();
        betButton = new javax.swing.JButton();
        checkButton = new javax.swing.JButton();
        betAmount = new javax.swing.JTextField();
        jScrollPane1 = new javax.swing.JScrollPane();
        statusText = new javax.swing.JTextPane();
        startButton = new javax.swing.JButton();
        jLayeredPane1 = new javax.swing.JLayeredPane();
        commCard5 = new javax.swing.JLabel();
        commCard4 = new javax.swing.JLabel();
        commCard3 = new javax.swing.JLabel();
        commCard2 = new javax.swing.JLabel();
        commCard1 = new javax.swing.JLabel();
        p1bet = new javax.swing.JTextPane();
        p2bet = new javax.swing.JTextPane();
        p3bet = new javax.swing.JTextPane();
        p4bet = new javax.swing.JTextPane();
        potAmount = new javax.swing.JTextPane();
        potLabel = new javax.swing.JTextPane();
        tableBackground = new javax.swing.JLabel();
        Player2 = new javax.swing.JTextPane();
        Player3 = new javax.swing.JTextPane();
        Player1 = new javax.swing.JTextPane();
        Player4 = new javax.swing.JTextPane();
        player1Chips = new javax.swing.JTextPane();
        player2Chips = new javax.swing.JTextPane();
        player3Chips = new javax.swing.JTextPane();
        player4Chips = new javax.swing.JTextPane();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        p1c1.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));

        p1c2.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));

        p2c1.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));

        p2c2.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));

        p3c1.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));

        p3c2.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));

        p4c1.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));

        p4c2.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));

        foldButton.setText("Fold");
        foldButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                foldButtonMouseReleased(evt);
            }
        });

        callButton.setText("Call");
        callButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                callButtonMouseReleased(evt);
            }
        });

        betButton.setText("Bet");
        betButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                betButtonMouseReleased(evt);
            }
        });

        checkButton.setText("Check");
        checkButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                checkButtonMouseReleased(evt);
            }
        });

        statusText.setEditable(false);
        jScrollPane1.setViewportView(statusText);

        startButton.setText("Start Hand");
        startButton.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                startButtonMouseReleased(evt);
            }
        });

        commCard5.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/commCards/back.jpg"));
        commCard5.setBounds(330, 120, 50, 63);
        jLayeredPane1.add(commCard5, javax.swing.JLayeredPane.DEFAULT_LAYER);

        commCard4.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/commCards/back.jpg"));
        commCard4.setBounds(270, 120, 50, 63);
        jLayeredPane1.add(commCard4, javax.swing.JLayeredPane.DEFAULT_LAYER);

        commCard3.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/commCards/back.jpg"));
        commCard3.setBounds(210, 120, 50, 63);
        jLayeredPane1.add(commCard3, javax.swing.JLayeredPane.DEFAULT_LAYER);

        commCard2.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/commCards/back.jpg"));
        commCard2.setBounds(150, 120, 50, 63);
        jLayeredPane1.add(commCard2, javax.swing.JLayeredPane.DEFAULT_LAYER);

        commCard1.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/commCards/back.jpg"));
        commCard1.setBounds(90, 120, 50, 63);
        jLayeredPane1.add(commCard1, javax.swing.JLayeredPane.DEFAULT_LAYER);

        p1bet.setEditable(false);
        p1bet.setBounds(20, 140, 60, 21);
        jLayeredPane1.add(p1bet, javax.swing.JLayeredPane.DEFAULT_LAYER);

        p2bet.setEditable(false);
        p2bet.setBounds(200, 10, 60, 21);
        jLayeredPane1.add(p2bet, javax.swing.JLayeredPane.DEFAULT_LAYER);

        p3bet.setEditable(false);
        p3bet.setBounds(390, 140, 60, 21);
        jLayeredPane1.add(p3bet, javax.swing.JLayeredPane.DEFAULT_LAYER);

        p4bet.setEditable(false);
        p4bet.setBounds(200, 270, 60, 21);
        jLayeredPane1.add(p4bet, javax.swing.JLayeredPane.DEFAULT_LAYER);

        potAmount.setEditable(false);
        potAmount.setBounds(190, 190, 90, 21);
        jLayeredPane1.add(potAmount, javax.swing.JLayeredPane.DEFAULT_LAYER);

        potLabel.setEditable(false);
        potLabel.setText("Pot:");
        potLabel.setBounds(160, 190, 30, 21);
        jLayeredPane1.add(potLabel, javax.swing.JLayeredPane.DEFAULT_LAYER);

        tableBackground.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/table.gif"));
        tableBackground.setBounds(-20, 0, 480, 300);
        jLayeredPane1.add(tableBackground, javax.swing.JLayeredPane.DEFAULT_LAYER);

        Player2.setEditable(false);
        Player2.setText("Player2");

        Player3.setEditable(false);
        Player3.setText("Player3");

        Player1.setEditable(false);
        Player1.setText("Player1");

        Player4.setEditable(false);
        Player4.setText("Player4");

        player1Chips.setEditable(false);

        player2Chips.setEditable(false);

        player3Chips.setEditable(false);

        player4Chips.setEditable(false);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(layout.createSequentialGroup()
                        .add(p1c1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(p1c2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(layout.createSequentialGroup()
                        .add(Player1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(player1Chips)))
                .add(34, 34, 34)
                .add(jLayeredPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 468, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(26, 26, 26)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(layout.createSequentialGroup()
                        .add(p3c1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(p3c2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(layout.createSequentialGroup()
                        .add(Player3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(player3Chips)))
                .addContainerGap(32, Short.MAX_VALUE))
            .add(layout.createSequentialGroup()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING, false)
                    .add(layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 362, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .add(foldButton))
                    .add(org.jdesktop.layout.GroupLayout.LEADING, layout.createSequentialGroup()
                        .add(349, 349, 349)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(layout.createSequentialGroup()
                                .add(Player4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(player4Chips))
                            .add(layout.createSequentialGroup()
                                .add(p4c1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(p4c2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(checkButton)
                .add(12, 12, 12)
                .add(callButton)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 12, Short.MAX_VALUE)
                .add(betButton)
                .add(18, 18, 18)
                .add(betAmount, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 85, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(50, 50, 50))
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap(351, Short.MAX_VALUE)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(layout.createSequentialGroup()
                        .add(Player2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(player2Chips))
                    .add(layout.createSequentialGroup()
                        .add(p2c1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(p2c2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 72, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .add(260, 260, 260)
                .add(startButton)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(251, 251, 251)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(p1c1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(p1c2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(player1Chips, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(Player1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(layout.createSequentialGroup()
                        .addContainerGap()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                    .add(p2c1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(p2c2))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                                    .add(Player2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(player2Chips, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 13, Short.MAX_VALUE)
                                .add(jLayeredPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 307, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(layout.createSequentialGroup()
                                .add(startButton)
                                .add(216, 216, 216)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                                    .add(p3c1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(p3c2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(Player3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(player3Chips, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(p4c1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(p4c2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 90, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(Player4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(player4Chips, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(35, 35, 35)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(betButton)
                            .add(foldButton)
                            .add(callButton)
                            .add(checkButton)
                            .add(betAmount, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 27, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .add(25, 25, 25))
                    .add(layout.createSequentialGroup()
                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 38, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap())))
        );
        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void betButtonMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_betButtonMouseReleased
        double desiredBet = new Double(betAmount.getText());
        if(table.getCurrentBet() == 0 && desiredBet < pokerGame.getBigBlind())
            return;
        if(desiredBet < table.getCurrentBet())
            return;
        makeBet(desiredBet, false);
    }//GEN-LAST:event_betButtonMouseReleased

    private void checkButtonMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_checkButtonMouseReleased
        //TablePot tPot = getTable().getActivePot();
        //Player player = getCurrentPlayer();
        //tPot.contributionChecked(player);
        TablePot tableP = getTable().getActivePot();
        if(tableP == null) {
            System.out.println("Error: No active pot, there should always be an active pot!");
            return;
        }
        PotContribution potC = tableP.getPlayerContribution(getCurrentPlayer());
        if(potC != null) {
            if(potC.getAmount() != getTable().getCurrentBet())
                return;
        } else if(getTable().getCurrentBet() > 0) {
            return;
        }
        makeBet(0, true);  //a check is just a 0 bet
    }//GEN-LAST:event_checkButtonMouseReleased
    
    private void makeBet(double amount, boolean checked) {
        Table table = getTable();
        TablePot currentPot = table.getActivePot();
        Player player = getCurrentPlayer();
        PotContribution existingC = currentPot.getPlayerContribution(player);
        PotContribution newC;
        double totalAmount;
        if(existingC == null || checked) {
            newC = player.bet(amount, false);
            totalAmount = newC.getAmount();
        } else {
            newC = player.bet(amount - existingC.getAmount(), false);
            totalAmount = newC.getAmount() + existingC.getAmount();  //in case the newC was an all in that couldn't match the currentBet
        }
        JTextPane textPane = betTexts.get(playersTurn);
        textPane.setText(doubleTwoDecimalPlaces(totalAmount));
        table.addPotContribution(newC, true);    
    }
    
    private void callButtonMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_callButtonMouseReleased
        makeBet(table.getCurrentBet(), false);
    }//GEN-LAST:event_callButtonMouseReleased

    private void foldButtonMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_foldButtonMouseReleased
        Player player = getCurrentPlayer();
        
        JLabel muckCard1 = pCards.get(playersTurn);
        JLabel muckCard2 = pCards.get(playersTurn + players.size());
        muckCard1.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));
        muckCard2.setIcon(new javax.swing.ImageIcon("/home/cosc/student/ejf31/Cosc427/Cards/back.jpg"));
        
        JTextPane textPane = betTexts.get(playersTurn);
        textPane.setText("Mucked");
        getTable().muckAllContributions(player);
        player.muckPlayerHand();
        int activeHands = getTable().getNumActivePlayerHands();
        if(activeHands == 1) {
            getTable().getDealer().endHand();
        }
    }//GEN-LAST:event_foldButtonMouseReleased

    private void resetHand() {
        clearStatusText();
        clearBetTexts();
        clearPotDisplay();
        afterFlop = false;
        handStarted = false;
        commCardCounter = 0;
    }
    
    
    private void startButtonMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_startButtonMouseReleased
        startButton.setEnabled(false);
        resetHand();
        new Thread(pokerGame).start();
    }//GEN-LAST:event_startButtonMouseReleased
    
    public void enableStartButton() {
        startButton.setEnabled(true);
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new TableGui().setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextPane Player1;
    private javax.swing.JTextPane Player2;
    private javax.swing.JTextPane Player3;
    private javax.swing.JTextPane Player4;
    private javax.swing.JTextField betAmount;
    private javax.swing.JButton betButton;
    private javax.swing.JButton callButton;
    private javax.swing.JButton checkButton;
    private javax.swing.JLabel commCard1;
    private javax.swing.JLabel commCard2;
    private javax.swing.JLabel commCard3;
    private javax.swing.JLabel commCard4;
    private javax.swing.JLabel commCard5;
    private javax.swing.JButton foldButton;
    private javax.swing.JLayeredPane jLayeredPane1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextPane p1bet;
    private javax.swing.JLabel p1c1;
    private javax.swing.JLabel p1c2;
    private javax.swing.JTextPane p2bet;
    private javax.swing.JLabel p2c1;
    private javax.swing.JLabel p2c2;
    private javax.swing.JTextPane p3bet;
    private javax.swing.JLabel p3c1;
    private javax.swing.JLabel p3c2;
    private javax.swing.JTextPane p4bet;
    private javax.swing.JLabel p4c1;
    private javax.swing.JLabel p4c2;
    private javax.swing.JTextPane player1Chips;
    private javax.swing.JTextPane player2Chips;
    private javax.swing.JTextPane player3Chips;
    private javax.swing.JTextPane player4Chips;
    private javax.swing.JTextPane potAmount;
    private javax.swing.JTextPane potLabel;
    private javax.swing.JButton startButton;
    private javax.swing.JTextPane statusText;
    private javax.swing.JLabel tableBackground;
    // End of variables declaration//GEN-END:variables
    
}
