import java.util.*;


public class TablePot extends Observable {
    
    private List<PotContribution> contributions;
    private boolean closed;
    private boolean playerAllIn;
        
    public TablePot() {
        contributions = new ArrayList<PotContribution>();
        setClosed(false);
        setPlayerAllIn(false);
    }
    
    public void addContribution(PotContribution pCon, boolean notify) {
        int index = contributions.indexOf(pCon);
        if(index < 0) {
            contributions.add(pCon);
        } else {
            PotContribution matchC = contributions.get(index);
            matchC.setAmount(pCon.getAmount() + matchC.getAmount());
            matchC.setAllIn(pCon.getAllIn());
            matchC.setForced(pCon.getForced());
        }    
        if(pCon.getAllIn())
            setPlayerAllIn(true);        
        if(notify) {
            setChanged();
            notifyObservers();
        }
    }
    
    //returns the current pot contribution for that player (if incomplete)
    public PotContribution getPlayerContribution(Player player) {
        for(int i = 0; i < contributions.size(); i++) {
            PotContribution potC = contributions.get(i);
            Player tempP = potC.getPlayer();
            if(tempP.equals(player) && !potC.getComplete())
                return potC;     
        }
        return null;
    }
    
    //returns a list of all pot contributions for that player including closed contributions
    public List<PotContribution> getAllPlayerContributions(Player player) {
        List<PotContribution> potConts = getContributions();
        List<PotContribution> playerConts = new ArrayList<PotContribution>();
        for(int i = 0; i < potConts.size(); i++) {
            PotContribution potC = potConts.get(i);
            if(potC.getPlayer().equals(player))
                playerConts.add(potC);            
        }        
        return playerConts;
    }
    
    //sets the forced value to false for the PotContribution from that Player (inidcating they have checked a forced bet)
    public void contributionChecked(Player player) {
        PotContribution potC = getPlayerContribution(player);
        if(potC == null) {
            System.out.println("Cannot check a non-existant bet!");
        } else {
            potC.setForced(false);
            setChanged();
            notifyObservers();
        }
    }
    
    public void muckContributions(Player player) {
        List<PotContribution> potConts = getAllPlayerContributions(player);
        for(int i = 0; i < potConts.size(); i++) {
            PotContribution potC = potConts.get(i);
            potC.setMucked(true);
        }
        setChanged();
        notifyObservers();
    }
    
    //return a list of contributions that aren't folded or all in, and that belong to that round (complete = false)
    public List<PotContribution> getActiveContributions() {
        List<PotContribution> allConts = getContributions();
        List<PotContribution> activeConts = new ArrayList<PotContribution>();
        for(int i = 0; i < allConts.size(); i++) {
            PotContribution potC = allConts.get(i);
            if(!potC.getAllIn() && !potC.getMucked() && !potC.getComplete())
                activeConts.add(potC);
        }
        return activeConts;
    }
    
    public boolean getClosed() {
        return closed;
    }
    
    public void setClosed(boolean closed) {
        this.closed = closed;
    }
    
    public boolean getPlayerAllIn() {
        return playerAllIn;
    }
    
    public void setPlayerAllIn(boolean allIn) {
        playerAllIn = allIn;
    }
    
    public List<PotContribution> getContributions() {
        return contributions;
    }
    
    //Dealer calls to set all PotContributions to complete to signify round ended
    public void setContributionsComplete() {
        List<PotContribution> conts = getContributions();
        for(int i = 0; i < conts.size(); i++) {
            PotContribution potC = conts.get(i);
            potC.setComplete(true);
        }        
    }
    
    //returns true if pot is complete (all active players have acted (forced == false) and have matched bets, or are all in, or are folded)
    public boolean contributionsMatched(int activePlayers) {
        List<PotContribution> activeConts = getActiveContributions();
        if(activeConts.size() < activePlayers)
            return false;
        double lastBet = 0;
        for(int i = 0; i < activeConts.size(); i++) {
            PotContribution currentCont = activeConts.get(i);
            if(currentCont.getForced())
                return false;
            if(currentCont.getAmount() != lastBet && i != 0)
                return false;           
            lastBet = currentCont.getAmount();
        }
        return true;
    }
    
    public double getTotal() {
        double total = 0;
        for(int i = 0; i < contributions.size(); i++) {
            PotContribution tempCont = contributions.get(i);
            total += tempCont.getAmount();            
        }
        return total;
    }
}
